import { ValueType, CollectedTypes } from '@previewjs/type-analyzer';
import ts from 'typescript';

declare type SerializableValue = SerializableArrayValue | SerializableBooleanValue | SerializableFunctionValue | SerializableMapValue | SerializableNullValue | SerializableNumberValue | SerializableObjectValue | SerializablePromiseValue | SerializableSetValue | SerializableStringValue | SerializableUndefinedValue | SerializableUnknownValue;
declare type SerializableArrayValue = {
    kind: "array";
    items: SerializableValue[];
};
declare function array(items: SerializableValue[]): SerializableArrayValue;
declare const EMPTY_ARRAY: SerializableArrayValue;
declare type SerializableBooleanValue = {
    kind: "boolean";
    value: boolean;
};
declare function boolean(value: boolean): SerializableBooleanValue;
declare const FALSE: SerializableBooleanValue;
declare const TRUE: SerializableBooleanValue;
declare type SerializableFunctionValue = {
    kind: "function";
    returnValue: SerializableValue;
};
declare function fn(returnValue: SerializableValue): SerializableFunctionValue;
declare type SerializableMapValue = {
    kind: "map";
    values: SerializableObjectValue;
};
declare function map(values: SerializableObjectValue): SerializableMapValue;
declare const NULL: {
    readonly kind: "null";
};
declare type SerializableNullValue = typeof NULL;
declare type SerializableNumberValue = {
    kind: "number";
    value: number;
};
declare function number(value: number): SerializableNumberValue;
declare type SerializableObjectValue = {
    kind: "object";
    entries: SerializableObjectValueEntry[];
};
declare type SerializableObjectValueEntry = {
    key: SerializableValue;
    value: SerializableValue;
};
declare function object(entries: SerializableObjectValueEntry[]): SerializableObjectValue;
declare type SerializablePromiseValue = {
    kind: "promise";
    value: {
        type: "reject";
        message: string | null;
    } | {
        type: "resolve";
        value: SerializableValue;
    };
};
declare function promise(value: {
    type: "reject";
    message: string | null;
} | {
    type: "resolve";
    value: SerializableValue;
}): SerializablePromiseValue;
declare const EMPTY_OBJECT: SerializableObjectValue;
declare const EMPTY_MAP: SerializableMapValue;
declare type SerializableSetValue = {
    kind: "set";
    values: SerializableArrayValue;
};
declare function set(values: SerializableArrayValue): SerializableSetValue;
declare const EMPTY_SET: SerializableSetValue;
declare type SerializableStringValue = {
    kind: "string";
    value: string;
};
declare function string(value: string): SerializableStringValue;
declare const UNDEFINED: {
    readonly kind: "undefined";
};
declare type SerializableUndefinedValue = typeof UNDEFINED;
declare type SerializableUnknownValue = {
    kind: "unknown";
    source?: string;
};
declare function unknown(source?: string): SerializableUnknownValue;
declare const UNKNOWN: SerializableUnknownValue;

/**
 * Generates a valid value for the given type.
 */
declare function generateSerializableValue(type: ValueType, collected: CollectedTypes, isFunctionReturnValue?: boolean, path?: string[]): SerializableValue;

declare function parseSerializableValue(expression: ts.Expression): SerializableValue;

declare function serializableValueToJavaScript(value: SerializableValue): string;

export { EMPTY_ARRAY, EMPTY_MAP, EMPTY_OBJECT, EMPTY_SET, FALSE, NULL, SerializableArrayValue, SerializableBooleanValue, SerializableFunctionValue, SerializableMapValue, SerializableNullValue, SerializableNumberValue, SerializableObjectValue, SerializableObjectValueEntry, SerializablePromiseValue, SerializableSetValue, SerializableStringValue, SerializableUndefinedValue, SerializableUnknownValue, SerializableValue, TRUE, UNDEFINED, UNKNOWN, array, boolean, fn, generateSerializableValue, map, number, object, parseSerializableValue, promise, serializableValueToJavaScript, set, string, unknown };
